<?php

namespace FiloBlu\Job\Controller\Send;

use Exception;
use FiloBlu\Job\Helper\Mailer;
use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\App\ResponseInterface;
use Magento\Framework\Controller\ResultInterface;
use Magento\Framework\DataObjectFactory;
use Magento\Framework\Exception\NotFoundException;
use Magento\Framework\Filesystem\Driver\File;
use Magento\Framework\Translate\Inline\StateInterface;
use Magento\Framework\View\Result\Page;
use Magento\Framework\View\Result\PageFactory;
use Magento\Store\Model\ScopeInterface;
use Psr\Log\LoggerInterface;
use Throwable;
use Zend_Validate_Interface;

/**
 * Class Index
 * @package FiloBlu\Job\Controller
 */
class Index extends Action
{
    /**
     * Enabled config path
     */
    const XML_PATH_ENABLED = 'filoblu_job_section/general/enabled';
    /**
     * @var StateInterface
     */
    protected $inlineTranslation;
    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;
    /**
     * @var File
     */
    protected $file;
    /**
     * @var LoggerInterface
     */
    protected $logger;
    /**
     * @var PageFactory
     */
    protected $pageFactory;
    /**
     * @var Zend_Validate_Interface
     */
    protected $validator;
    /**
     * @var string
     */
    protected $attachmentName;
    /**
     * @var DataObjectFactory
     */
    protected $dataObjectFactory;
    /**
     * @var Mailer
     */
    protected $mailer;

    /**
     * Index constructor.
     * @param Context $context
     * @param StateInterface $inlineTranslation
     * @param ScopeConfigInterface $scopeConfig
     * @param File $file
     * @param PageFactory $pageFactory
     * @param Mailer $mailer
     * @param Zend_Validate_Interface $validator
     * @param LoggerInterface $logger
     * @param DataObjectFactory $dataObjectFactory
     * @param $attachmentName
     */
    public function __construct(
        Context $context,
        StateInterface $inlineTranslation,
        ScopeConfigInterface $scopeConfig,
        File $file,
        PageFactory $pageFactory,
        Mailer $mailer,
        Zend_Validate_Interface $validator,
        LoggerInterface $logger,
        DataObjectFactory $dataObjectFactory,
        $attachmentName
    ) {
        parent::__construct($context);
        $this->inlineTranslation = $inlineTranslation;
        $this->scopeConfig = $scopeConfig;
        $this->file = $file;
        $this->logger = $logger;
        $this->validator = $validator;
        $this->mailer = $mailer;
        $this->pageFactory = $pageFactory;
        $this->attachmentName = $attachmentName;
        $this->dataObjectFactory = $dataObjectFactory;
    }

    /**
     * Dispatch request
     *
     * @param RequestInterface $request
     * @return ResponseInterface
     * @throws NotFoundException
     */
    public function dispatch(RequestInterface $request)
    {
        if ($this->scopeConfig->isSetFlag(self::XML_PATH_ENABLED, ScopeInterface::SCOPE_STORE)) {
            return parent::dispatch($request);
        }

        throw new NotFoundException(__('Page not found.'));
    }

    /**
     * @return ResponseInterface|ResultInterface|Page
     */
    public function execute()
    {
        $request = $this->getRequest();
        $post = $request->getPostValue();
        $attachment = null;
        if (!$post && isset($post['job_submit'])) {
            return $this->pageFactory->create();
        }

        try {
            $this->inlineTranslation->suspend();
            if (!isset($post['interestArea'])) {
                $post['interestArea'] = '';
            }

            if (!isset($post['country'])) {
                $post['country'] = '';
            }

            if ($this->validator->isValid($request)) {
                $parameters = $request->getFiles($this->attachmentName);
                $attachment = $parameters['tmp_name'];

                $this->mailer->sendMail(
                    $this->dataObjectFactory->create(['data' => $post]),
                    $post['email'],
                    $attachment,
                    $parameters['name']
                );
            } else {
                $this->messageManager->addErrorMessage(
                    implode(PHP_EOL, $this->validator->getMessages())
                );
            }
        } catch (Exception $exception) {
            $this->messageManager->addErrorMessage($exception->getMessage());
            $this->logger->error($exception->getMessage(), ['exception' => $exception]);
        } catch (Throwable $throwable) {
            $this->messageManager->addErrorMessage($throwable->getMessage());
            $this->logger->error($throwable->getMessage(), ['exception' => $throwable]);
        }

        $this->inlineTranslation->resume();

        try {
            if ($attachment) {
                $this->file->deleteFile($attachment);
            }
        } catch (Exception $exception) {
            $this->logger->error($exception->getMessage(), ['exception' => $exception]);
        }

        $this->messageManager->addSuccessMessage(__('Thanks for contacting us. We\'ll respond to you very soon.'));
        return $this->_redirect($this->_redirect->getRedirectUrl());
    }
}
